from textual.widget import Widget
from textual.widgets import Static

from txl.base import MainArea as AbstractMainArea


class MainAreaMeta(type(Widget), type(AbstractMainArea)):
    pass


class MainArea(Widget, AbstractMainArea, metaclass=MainAreaMeta):

    def __init__(self):
        super().__init__(id="main-view")
        self.mounter = None

    def init(self):
        self.mounted = Static()
        super().mount(self.mounted)

    def mount(self, widget):
        if self.mounted:
            self.mounted.remove()
        self.mounted = widget
        super().mount(widget)



from __future__ import annotations

from collections import defaultdict

import rich.repr
from rich.console import RenderableType
from rich.text import Text

from textual import events
from textual.reactive import Reactive, watch
from textual.widget import Widget

from txl.base import Footer as AbstractFooter


class FooterMeta(type(AbstractFooter), type(Widget)):
    pass


@rich.repr.auto
class Footer(AbstractFooter, Widget, metaclass=FooterMeta):
    """A simple header widget which docks itself to the top of the parent container."""

    DEFAULT_CSS = """
    Footer {
        background: $accent;
        color: $text;
        dock: bottom;
        height: 1;
    }
    Footer > .footer--highlight {
        background: $accent-darken-1;
    }

    Footer > .footer--highlight-key {
        background: $secondary;
        text-style: bold;
    }

    Footer > .footer--key {
        text-style: bold;
        background: $accent-darken-2;
    }
    """

    COMPONENT_CLASSES = {
        "footer--description",
        "footer--key",
        "footer--highlight",
        "footer--highlight-key",
    }

    def __init__(self) -> None:
        super().__init__()
        self._key_text: Text | None = None
        self._bindings = None
        self.auto_links = False

    highlight_key: Reactive[str | None] = Reactive(None)

    async def watch_highlight_key(self, value) -> None:
        """If highlight key changes we need to regenerate the text."""
        self._key_text = None

    def on_mount(self) -> None:
        watch(self.screen, "focused", self._focus_changed)

    def _focus_changed(self, focused: Widget | None) -> None:
        self._key_text = None
        self.refresh()

    async def on_mouse_move(self, event: events.MouseMove) -> None:
        """Store any key we are moving over."""
        self.highlight_key = event.style.meta.get("key")

    async def on_leave(self, event: events.Leave) -> None:
        """Clear any highlight when the mouse leaves the widget"""
        self.highlight_key = None

    def __rich_repr__(self) -> rich.repr.Result:
        yield from super().__rich_repr__()

    def make_key_text(self) -> Text:
        """Create text containing all the keys."""
        bindings = self._bindings
        base_style = self.rich_style
        text = Text(
            style=self.rich_style,
            no_wrap=True,
            overflow="ellipsis",
            justify="left",
            end="",
        )
        highlight_style = self.get_component_rich_style("footer--highlight")
        highlight_key_style = self.get_component_rich_style("footer--highlight-key")
        key_style = self.get_component_rich_style("footer--key")

        if bindings is None:
            bindings = [
                binding
                for (_namespace, binding) in self.app.namespace_bindings.values()
                if binding.show
            ]

        action_to_bindings = defaultdict(list)
        for binding in bindings:
            action_to_bindings[binding.action].append(binding)

        for action, bindings in action_to_bindings.items():
            binding = bindings[0]
            key_display = (
                binding.key.upper()
                if binding.key_display is None
                else binding.key_display
            )
            hovered = self.highlight_key == binding.key
            key_text = Text.assemble(
                (f" {key_display} ", highlight_key_style if hovered else key_style),
                (
                    f" {binding.description} ",
                    highlight_style if hovered else base_style,
                ),
                meta={
                    "@click": f"app.check_bindings('{binding.key}')" if self._bindings is None else binding.action,
                    "key": binding.key,
                },
            )
            text.append_text(key_text)
        return text

    def update_bindings(self, bindings) -> None:
        self._bindings = bindings
        self._key_text = self.make_key_text()
        self.refresh()

    def post_render(self, renderable):
        return renderable

    def render(self) -> RenderableType:
        if self._key_text is None:
            self._key_text = self.make_key_text()
        return self._key_text



from asphalt.core import Component, Context
from textual.app import App, ComposeResult
from textual.containers import Container
from textual.reactive import var
from txl.base import FileBrowser, Terminals, Footer, Header, MainArea
from txl.hooks import register_component
from txl_editors import Editors

from .footer import Footer as _Footer
from .header import Header as _Header
from .main_area import MainArea as _MainArea


class Jpterm(App):

    CSS_PATH = "jpterm.css"
    BINDINGS = [
        ("f", "toggle_files", "Toggle Files"),
        ("t", "open_terminal", "Open Terminal"),
        ("q", "quit", "Quit"),
    ]
    show_browser = var(True)

    def __init__(self, header, footer, main_area, file_browser, editors, terminals, *args, **kwargs):
        self.header = header
        self.footer = footer
        self.main_area = main_area
        self.file_browser = file_browser
        self.editors = editors
        self.terminals = terminals
        super().__init__(*args, **kwargs)

    def watch_show_browser(self, show_browser: bool) -> None:
        self.set_class(show_browser, "-show-browser")

    def compose(self) -> ComposeResult:
        yield self.header
        self.main_area.init()
        yield Container(
            self.file_browser,
            self.main_area,
        )
        yield self.footer

    def action_toggle_files(self) -> None:
        self.show_browser = not self.show_browser

    async def action_open_terminal(self) -> None:
        terminal = self.terminals()
        self.main_area.mount(terminal)
        await terminal.open()


class JptermComponent(Component):

    async def start(
        self,
        ctx: Context,
    ) -> None:
        header = _Header()
        footer = _Footer()
        main_area = _MainArea()
        ctx.add_resource(header, name="header", types=Header)
        ctx.add_resource(footer, name="footer", types=Footer)
        ctx.add_resource(main_area, name="main_area", types=MainArea)
        file_browser = await ctx.request_resource(FileBrowser, "file_browser")
        editors = await ctx.request_resource(Editors, "editors")
        terminals = await ctx.request_resource(Terminals, "terminals")
        file_browser.open_file_signal.connect(editors.on_open)
        jpterm = Jpterm(
            header,
            footer,
            main_area,
            file_browser,
            editors,
            terminals,
        )
        ctx.add_resource(jpterm, name="app", types=App)


c = register_component("app", JptermComponent)
